/*      Author: Rudolf Marek, Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda pot Fourierovu transformaci z vektoru komplexnch sel.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Rudolf Marek, Jan Dolezel
 * @version 1.2.0
 */
public class FFT {
  private FFT() {}

  /**
   * Vypocita Fourierovu transformaci vektoru komplexnich cisel. Pocet vzorku je nejblizsi vyssi mocnina dvou.
   * @param data ComplexVector
   * @return ComplexVector
   */
  static public ComplexVector fft(ComplexVector data) {
    //pocet samplu zaokrouhleny na mocniny 2
    int numSamples = (2 << (numberOfBitsNeeded(data.length()) - 1));
    return fft(data, numSamples, false);
  }

  /**
   * Vypocita Fourierovu transformaci vektoru komplexnich cisels danym poctem vzorku.
   * @param data ComplexVector
   * @param numSamples int
   * @return ComplexVector
   */
  static public ComplexVector fft(ComplexVector data, int numSamples) {
    return fft(data, numSamples, false);
  }

  /**
   * Vypocita inverzni Fourierovu transformaci vektoru komplexnich cisel. Pocet vzorku je nejblizsi vyssi mocnina dvou.
   * @param data ComplexVector
   * @return ComplexVector
   */
  static public ComplexVector ifft(ComplexVector data) {
    //pocet samplu zaokrouhleny na mocniny 2
    int numSamples = (2 << (numberOfBitsNeeded(data.length()) - 1));
    return fft(data, numSamples, true);
  }

  /** jadro provadejici Fourierovu tranformaci (i zpetnou)
   * vstupem je skutecny pocet vzorku, rnumSamples
   * o jaky typ transformace se jedna true = zpetna
   * @param data ComplexVector vstupni data
   * @param numSamples int pocet vzorku
   * @param inverseTransform boolean urcuje jestli budeme delat FFT nebo IFFT
   * @return ComplexVector
   */
  static public ComplexVector fft(ComplexVector data, int numSamples,
                                  boolean inverseTransform) {
    if (numSamples < (2 << (numberOfBitsNeeded(data.length()) - 1))) {
      throw new Error("Delka fft mensi nez pocet vzorku");
    }

    ComplexVector res = new ComplexVector(numSamples);
    int numBits = numberOfBitsNeeded(numSamples); //jak velke pole budeme potrebovat
    double angle_numerator = 2.0 * Math.PI;

    if (inverseTransform == true) {
      angle_numerator = -angle_numerator; //prehodime 2PI na -2PI
    }

    //zkopirujeme do vysledneho pole, zbytek doplnime nulama
    //rovnou rozradime, sampl nula bude na nule prvni v pulce, druhy ve ctvrtine
    // 3 v 3/4, 4 v 1/8 atd...
    // nutne pro Radix-2 algoritmus, rozdel a panuj
    //
    // zbytek doplnime nulama

    for (int i = 0; i < numSamples; i++) {
      int j = reverseBits(i, numBits);
      if (i < data.length()) {
        res.set(j, new Complex(data.get(i)));
      }
      else {
        res.set(j, new Complex());
      }
    }

    // Nerekurzivni Radix-2 algorimus
    // http://www.eptools.com/tn/T0001/PT04.HTM

    int blockEnd = 1;
    for (int blockSize = 2; blockSize <= numSamples; blockSize <<= 1) {
      // delta_angle budeme rotovat fazor usetrime vypocet sin, cos

      double delta_angle = angle_numerator / (double) blockSize;
      double sm2 = Math.sin( -2 * delta_angle);
      double sm1 = Math.sin( -delta_angle);
      double cm2 = Math.cos( -2 * delta_angle);
      double cm1 = Math.cos( -delta_angle);
      double w = 2 * cm1;
      Complex A0, A1, A2;

      for (int i = 0; i < numSamples; i += blockSize) {
        A1 = new Complex(cm1, sm1);
        A2 = new Complex(cm2, sm2);
        for (int j = i, n = 0; n < blockEnd; j++, n++) {
          A0 = new Complex(w * A1.getReal() - A2.getReal(),
                           w * A1.getIm() - A2.getIm());
          A2 = A1;
          A1 = A0;

          int k = j + blockEnd;
          Complex temp = A0.multiple(res.get(k));
          res.set(k, res.get(j).substract(temp));
          res.set(j, res.get(j).add(temp));
        }
      }
      blockEnd = blockSize;
    }

    if (inverseTransform == true) {
      double denom = (double) numSamples;
      for (int i = 0; i < numSamples; i++) {
        res.get(i).divide(denom);
      }
    }

    return res;
  }

  /**
   * vypocita ze zadaneho vstupu, nejblizsi mocninu cisla 2
   * @param num int cislo jehoz nejblizsi vetsi mocninu 2 chceme zjistit
   * @return int nejblizsi mocnina
   */
  static int numberOfBitsNeeded(int num) {
    return (int) Math.ceil(Math.log(num) / Math.log(2));
  }

  /**
   * vypocita nove umisteni prvku pole s urcitym indexem
   * pouziva interne fft - (vzorky se rozmistuji na nova mista
   * algoritmus pouziva paradigma rozdel a panuj)
   * @param index int jaky prvek
   * @param numBits int jaky rozsah
   * @return int nove umisteni
   */
  private static int reverseBits(int index, int numBits) {
    int rev = 0;
    for (int i = 0; i < numBits; i++) {
      rev = (rev << 1) | (index & 1);
      index >>= 1;
    }
    return rev;
  }
}
