/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda implementuje linern regresi dat.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class LinearRegression {
  protected double fi;
  protected double offset;
//  protected double[][] residuals = null;

  /**
   * Linern regrese dvourozmrnch dat uren poly x a y. Vsledek je pstupn metodami {@link #getFi() getFi()} a {@link #getOffset() getOffset()}
   * @param x int[]
   * @param y int[]
   */
  public LinearRegression(int x[], int y[]) {
    this(asDouble(x), asDouble(y));
  }

  /**
   * Peveden pole int na double
   * @param a int[]
   * @return double[]
   */
  protected static double[] asDouble(int a[]) {
    int len = a.length;
    double[] res = new double[len];
    for (int i = 0; i < len; i++) {
      res[i] = a[i];
    }
    return res;
  }

  /**
   * Linern regrese dvourozmrnch dat uren poly x a y. Vsledek je pstupn metodami {@link #getFi() getFi()} a {@link #getOffset() getOffset()}
   * @param x double[]
   * @param y double[]
   */
  public LinearRegression(double x[], double y[]) {
    int len = Math.min(x.length, y.length);
    double minX = Double.MAX_VALUE, minY = Double.MAX_VALUE;
    double maxX = Double.MIN_VALUE, maxY = Double.MIN_VALUE;
    for (int i = len - 1; i >= 0; i--) {
      minX = Math.min(minX, x[i]);
      minY = Math.min(minY, y[i]);
      maxX = Math.max(maxX, x[i]);
      maxY = Math.max(maxY, y[i]);
    }
    if ( (maxX - minX) > (maxY - minY)) {
      this.computeRegressionHorizontal(x, y);
    }
    else {
      this.computeRegressionVertical(x, y);
    }
    /*
        double [] sx = x;
        double [] sy = y;
        Arrays.sort(sx);
        Arrays.sort(sy);
        this.computeRegression(sx, sy);
     */
  }

  /**
   * Vlastn vpoet linern regrese. Pouv se v ppad, e body maj piblin stejnou vzdlenost
   * Pouv vzorec: fi = (Ey*Ei - Eyi) / (Ex*Ei - Exi)
   * @param x double[]
   * @param y double[]
   */
  protected void computeRegression(double x[], double y[]) {
    int len = Math.min(x.length, y.length);
    if (len > 0) {
      double sumX = 0, sumY = 0, sumXi = 0, sumYi = 0;
      for (int i = 0; i < len; i++) {
        sumX += x[i];
        sumY += y[i];
        sumXi += x[i] * i;
        sumYi += y[i] * i;
      }
      double kxi = sumX * (len + 1) / 2 - sumXi;
      double kyi = sumY * (len + 1) / 2 - sumYi;
      fi = kyi / kxi;
      offset = (sumY - fi * sumX) / len;
    }
  }

  /**
   * Vlastn vpoet linern regrese. Pouv se v ppad, e data jsou spe horizontln
   * pouv vzorec: fi = (Exy - Ex*Ey) / (Ex<sup>2</sup> - (Ex)<sup>2</sup>)
   * @param x double[]
   * @param y double[]
   */
  protected void computeRegressionHorizontal(double x[], double y[]) {
    int len = Math.min(x.length, y.length);
    if (len > 0) {
      double sumX = 0, sumY = 0, sumXX = 0, sumXY = 0, sumYY = 0;
      for (int i = 0; i < len; i++) {
        sumX += x[i];
        sumY += y[i];
        sumXX += x[i] * x[i];
        sumYY += y[i] * y[i];
        sumXY += x[i] * y[i];
      }
      double Sxx = sumXX - sumX * sumX / len;
      double Sxy = sumXY - sumX * sumY / len;
      fi = Sxy / Sxx;
      offset = (sumY - fi * sumX) / len;
    }
  }

  /**
   * Vlastn vpoet linern regrese. Pouv se v ppad, e data jsou spe vertikln
   * pouv vzorec: fi = Efi
   * @param x double[]
   * @param y double[]
   */
  protected void computeRegressionVertical(double x[], double y[]) {
    int len = Math.min(x.length, y.length);
    if (len > 0) {
      double sumX = 0, sumY = 0;
      double lx = x[0], ly = y[1];
      double length = 0;
      for (int i = 1; i < len; i++) {
        double ds = Math.sqrt( (ly - y[i]) * (ly - y[i]) +
                              (lx - x[i]) * (lx - x[i]));
        length += ds;
        fi += Math.atan2(ly - y[i], lx - x[i]) * ds;
        sumX += x[i];
        sumY += y[i];
        lx = x[i];
        ly = y[i];
      }
      fi = Math.tan(fi / length);
      offset = (sumY - fi * sumX) / len;
    }
  }

  /*
    protected void computeResiduals(double x[], double y[]) {
      int len = Math.min(x.length, y.length);
      residuals = new double[len][2];
      for (int i = 0; i < len; i++) {
        residuals[i][0] = x[i];
        residuals[i][1] = y[i] - (offset + fi * x[i]);
      }
    }
   */

  /**
   * Sklon pmky y = fi*x + offset
   * @return double
   */
  public double getFi() {
    return fi;
  }

  public double getOffset() {
    return offset;
  }
  /*
    public double[][] getResiduals() {
      return residuals;
    }
   */
}
