/*      Author: Miroslav Skrbek, Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda implementuje matici relnch sel.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Miroslav Skrbek, Jan Dolezel
 * @version 1.2.0
 */
public class Matrix {
  private int n;
  private int m;
  private double matrix[][];

  /**
   * Przdn matice 0x0
   */
  public Matrix() {
    this(0);
  }

  /**
   * tvercov matice n x n
   * @param n int
   */
  public Matrix(int n) {
    this(n, n);
  }

  public Matrix(int width, int height) {
    this.n = width;
    this.m = height;
    this.matrix = new double[m][n];
  }

  public Matrix(Matrix matrix) {
    this.n = matrix.n;
    this.m = matrix.m;
    this.matrix = new double[m][n];
    for (int i = 0; i < m; i++) {
      for (int j = 0; j < n; j++) {
        this.matrix[i][j] = matrix.matrix[i][j];
      }
    }
  }

  /**
   * vytv matici z dvourozmrnho pole relnch sel
   * @param matrix double[][]
   */
  public Matrix(double matrix[][]) {
    this.m = matrix.length;
    if (m > 0) {
      this.n = matrix[0].length;

      for (int i = 0; i < m; i++) {
        if (n != matrix[i].length) {
          throw new Error("Chyba v poli pri inicializaci matice");
        }
      }

      this.matrix = new double[m][n];
      for (int i = 0; i < m; i++) {
        for (int j = 0; j < n; j++) {
          this.matrix[i][j] = matrix[i][j];
        }
      }
    }
    else {
      this.m = 0;
      this.matrix = new double[0][0];
    }
  }

  /**
   * vrac prvek pole na pozici i,j
   * @param i int sloupec
   * @param j int dek
   * @return double
   */
  public double get(int i, int j) {
    return matrix[j][i];
  }

  /**
   * nastav prvek pole na pozici i,j
   * @param i int sloupec
   * @param j int dek
   * @param value double
   */
  public void set(int i, int j, double value) {
    matrix[j][i] = value;
  }

  /**
   * provede LU dekompozici matice
   * @return Matrix[]
   */
  public Matrix[] LUdecomposition() {
    int max = Math.max(n, m);
    Matrix aux = new Matrix(this);
    aux.reshape(max, max);
    Matrix l = new Matrix(max);
    Matrix u = new Matrix(max);

    for (int k = 0; k < max; k++) {
      u.matrix[k][k] = aux.matrix[k][k];
      l.matrix[k][k] = 1;
      for (int i = k + 1; i < max; i++) {
        l.matrix[i][k] = aux.matrix[i][k] / u.matrix[k][k];
        u.matrix[k][i] = aux.matrix[k][i];
      }
      for (int i = k + 1; i < max; i++) {
        for (int j = k + 1; j < max; j++) {
          aux.matrix[i][j] = aux.matrix[i][j] - l.matrix[i][k] * u.matrix[k][j];
        }
      }
    }

    int min = Math.min(n, m);
    l.reshape(min, m);
    u.reshape(n, min);
    Matrix[] res = {
        l, u};
    return res;
  }

  /**
   * vypote matici inverzn
   * @return Matrix
   */
  public Matrix inverse() {
    if (n != m) {
      throw new Error("Inverze pouze pro ctvercove matice!");
    }

    Matrix res = new Matrix(n);
    Matrix[] lu = this.LUdecomposition();
    Matrix l = lu[0];
    Matrix u = lu[1];

    for (int k = 0; k < n; k++) {
      double[] y = new double[n];
      for (int i = 0; i < n; i++) {
        double s = 0;
        for (int j = 0; j < i; j++) {
          s += l.matrix[i][j] * y[j];
        }
        y[i] = ( (i == k) ? 1 : 0) - s;
      }

      for (int i = n - 1; i >= 0; i--) {
        double s = 0;
        for (int j = i + 1; j < n; j++) {
          s += u.matrix[i][j] * res.matrix[j][k];
        }
        res.set(k, i, (y[i] - s) / u.matrix[i][i]);
      }
    }

    res.reshape(m, n);
    return res;
  }

  /**
   * nsoben matic - pvodn matice nech beze zmny
   * @param matrix Matrix
   * @return Matrix vsledn matice
   */
  public Matrix mulipleBy(Matrix matrix) {
    if (this.n != matrix.m) {
      throw new Error("Odpovidajici rozmery matic musi souhlasit");
    }

    Matrix res = new Matrix(matrix.n, this.m);
    for (int i = 0; i < this.m; i++) {
      for (int j = 0; j < matrix.n; j++) {
        double sum = 0;
        for (int k = 0; k < n; k++) {
          sum += this.matrix[i][k] * matrix.matrix[k][j];
        }
        res.matrix[i][j] = sum;
      }
    }
    return res;
  }

  /**
   * Nsoben matice vektorem. Nsobitel i nsobenec ponechv beze zmny
   * @param vector Vektor
   * @return Vektor
   */
  public Vektor mulipleBy(Vektor vector) {
    if (this.n != vector.getLength()) {
      throw new Error("Odpovidajici rozmery matic musi souhlasit");
    }

    Vektor res = new Vektor(this.m);
    for (int i = 0; i < m; i++) {
      double sum = 0;
      for (int k = 0; k < n; k++) {
        sum += this.matrix[i][k] * vector.get(k);
      }
      res.set(i, sum);
    }
    return res;
  }

  public String toString() {
    StringBuffer buf = new StringBuffer(m * (n + 1) * 3);
    for (int i = 0; i < m; i++) {
      buf.append(matrix[i][0]);
      for (int j = 1; j < n; j++) {
        buf.append(", ");
        buf.append(matrix[i][j]);
      }
      buf.append("\n");
    }
    return buf.toString();
  }

  /**
   * vrac transponovanou matici
   * @return Matrix
   */
  public Matrix transposed() {
    Matrix res = new Matrix(m, n);
    for (int i = 0; i < m; i++) {
      for (int j = 0; j < n; j++) {
        res.matrix[j][i] = matrix[i][j];
      }
    }
    return res;
  }

  public int getWidth() {
    return n;
  }

  public int getHeight() {
    return m;
  }

  public boolean equals(Object o) {
    if (o instanceof Matrix) {
      Matrix m = (Matrix) o;
      if ( (this.m != m.m) || (this.n != m.n)) {
        return false;
      }
      for (int i = 0; i < this.m; i++) {
        for (int j = 0; j < this.n; j++) {
          if (matrix[i][j] != m.matrix[i][j]) {
            return false;
          }
        }
      }
      return true;
    }
    return false;
  }

  /**
   * ozne i roz matici na novou velikost
   * @param width int
   * @param height int
   */
  public void reshape(int width, int height) {
    double[][] newMatrix = new double[height][width];
    int nn = Math.min(width, n);
    int nm = Math.min(height, m);
    for (int i = 0; i < nn; i++) {
      for (int j = 0; j < nm; j++) {
        newMatrix[j][i] = matrix[j][i];
      }
    }
    matrix = newMatrix;
    n = width;
    m = height;
  }

  /**
   * Vytv matici identity n x n - prvky na diagonle jsou 1
   * @param n int
   * @return Matrix
   */
  public static Matrix getIdentity(int n) {
    Matrix m = new Matrix(n);
    for (int i = 0; i < n; i++) {
      m.matrix[i][i] = 1;
    }
    return m;
  }
}
