/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data;

import spirals.motol.database.Data;
import tablet.*;
import java.util.*;
import mathutil.*;

/**
 * <p>Tda uchovv data nasnman z tabletu.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class TabletData {
  /**
   * Informace o nasnmanch datech
   */
  private Data data;
  /**
   * vlastn nasnman data
   */
  private List<TabletPoint> vektor = new ArrayList<TabletPoint> ();
  private int minx = Integer.MAX_VALUE;
  private int maxx = Integer.MIN_VALUE;
  private int miny = Integer.MAX_VALUE;
  private int maxy = Integer.MIN_VALUE;

  public TabletData(Data data) {
    this.data = data;
    TabletInfo ti = data.getTabletInfo();
    maxx = ti.getMinX();
    minx = ti.getMaxX();
    maxy = ti.getMinY();
    miny = ti.getMaxY();
  }

  /**
   * pid jeden bod na konec dat
   * @param tabletPoint TabletPoint
   */
  public void add(TabletPoint tabletPoint) {
    vektor.add(tabletPoint);
    minx = Math.min(minx, tabletPoint.getX());
    maxx = Math.max(maxx, tabletPoint.getX());
    miny = Math.min(miny, tabletPoint.getY());
    maxy = Math.max(maxy, tabletPoint.getY());
  }

  public TabletPoint get(int index) {
    return vektor.get(index);
  }

  public int size() {
    return vektor.size();
  }

  /**
   * odstan pebyten body (pero je zvednuto) z konce dat
   */
  public void trim() {
    for (int i = vektor.size() - 1; i >= 0; i--) {
      if (!vektor.get(i).isTouched()) {
        vektor.remove(i);
      }
      else {
        return;
      }
    }
  }

  public int getMinX() {
    return minx;
  }

  public int getMaxX() {
    return maxx;
  }

  public int getMinY() {
    return miny;
  }

  public int getMaxY() {
    return maxy;
  }

  /**
   * vrac pole x-ovch souadnic
   * @return int[]
   */
  public int[] getIntX() {
    int[] x = new int[vektor.size()];
    for (int i = vektor.size() - 1; i >= 0; i--) {
      x[i] = vektor.get(i).getX();
    }
    return x;
  }

  /**
   * vrac pole y-ovch souadnic
   * @return int[]
   */
  public int[] getIntY() {
    int[] y = new int[vektor.size()];
    for (int i = vektor.size() - 1; i >= 0; i--) {
      y[i] = vektor.get(i).getY();
    }
    return y;
  }

  /**
   * vrac pole x-ovch souadnic
   * @return double[]
   */
  public double[] getDoubleX() {
    double[] x = new double[vektor.size()];
    for (int i = vektor.size() - 1; i >= 0; i--) {
      x[i] = vektor.get(i).getX();
    }
    return x;
  }

  /**
   * vrac pole x-ovch souadnic
   * @return double[]
   */
  public double[] getDoubleY() {
    double[] y = new double[vektor.size()];
    for (int i = vektor.size() - 1; i >= 0; i--) {
      y[i] = vektor.get(i).getY();
    }
    return y;
  }

  /**
   * Vrac vlastn data
   * @return List
   */
  public List<TabletPoint> getData() {
    return vektor;
  }

  private LinearRegression lr = null;

  /**
   * Vrac sklon linern regrese. Platn pouze pro ry.
   * @return double
   */
  public double getFi() {
    if (lr == null) {
      lr = new LinearRegression(getDoubleX(), getDoubleY());
    }
    return lr.getFi();
  }

  /**
   * Vrac offset linern regrese. Platn pouze pro ry.
   * @return double
   */
  public double getOffset() {
    if (lr == null) {
      lr = new LinearRegression(getDoubleX(), getDoubleY());
    }
    return lr.getOffset();
  }

  /**
   * Nech ve vsledku jen body, kter jsou od sebe dl, ne uren vzdlenost
   * @param sample double vzdlenost mezi body
   * @return TabletData pevzorkovan data
   */
  public TabletData resampleWithThrowaway(double sample) {
    TabletData res = new TabletData(data);
    int size = vektor.size();
    if (size > 0) {
      TabletPoint old = vektor.get(0);
      res.add(old);
      for (int i = 1; i < size; i++) {
        TabletPoint tp = vektor.get(i);
        int dx = tp.getX() - old.getX();
        int dy = tp.getY() - old.getY();
        if (Math.sqrt( (dx * dx) + (dy * dy)) > sample) {
          old = tp;
          res.add(old);
        }
      }
      res.add(vektor.get(size - 1));
    }
    return res;
  }

  /**
   * Pevede data na posloupnost, kter m mezi body pesn stejn rozestupy.
   * Z funkce asu na funkci dlky
   * @param distance double vzdlenost mezi body
   * @return TabletData pevzorkovan data
   */
  public TabletData resampleEquidistant(double distance) {
    TabletData res = new TabletData(data);
    int size = vektor.size();
    if (size > 0) {
      TabletPoint old = vektor.get(0);
      res.add(old);
      double ds[] = new double[size];

      for (int i = size - 2; i > 0; i--) {
        double dx = (double) (vektor.get(i - 1).getX() - vektor.get(i + 1).getX()) /
            2;
        double dy = (double) (vektor.get(i - 1).getY() - vektor.get(i + 1).getY()) /
            2;
        ds[i] = Math.sqrt( (dx * dx) + (dy * dy));
      }

      double cds = 0;
      int i = 0;
      int sn = old.getSerialNumber();
      while (i < (size - 1)) {
        while ( (cds < distance) && (i < (size - 1))) {
          cds = cds + ds[++i];
        }
        while ( (cds >= distance) && (i < size)) {
          double a = 1 - (cds - distance) / ds[i];
          res.add(this.getLinearCombination(vektor.get(i - 1), vektor.get(i), a,
                                            ++sn));
          cds = cds - distance;
        }
      }
      res.add(vektor.get(size - 1));
    }
    return res;
  }

  /**
   * Pevede data na posloupnost, kter m mezi body pesn stejn rozestupy.
   * Funkce zahazuje body, kter maj men ne zadanou vzdlenost
   * @param distance double vzdlenost mezi body
   * @return TabletData pevzorkovan data
   */
  public TabletData resampleEquidistantWithThrowaway(double distance) {
    TabletData res = new TabletData(data);
    int size = vektor.size();
    if (size > 0) {
      TabletPoint old = vektor.get(0);
      res.add(old);

      double cds = 0, oldcds = 0;
      int i = 0;
      int j = 0;
      int sn = old.getSerialNumber();
      while (j < size) {
        while ( (cds < distance) && (j < size)) {
          j++;
          double dx = (double) (vektor.get(j).getX() - vektor.get(i).getX());
          double dy = (double) (vektor.get(j).getY() - vektor.get(i).getY());
          cds = oldcds + Math.sqrt( (dx * dx) + (dy * dy));
        }
        double dist = cds;
        while ( (cds >= distance) && (j < size)) {
          double a = 1 - (cds - distance) / dist;
          res.add(this.getLinearCombination(vektor.get(i), vektor.get(j), a,
                                            ++sn));
          cds = cds - distance;
        }
        oldcds = cds;
        i = j;
      }
      res.add(vektor.get(size - 1));
    }
    return res;
  }

  /**
   * Vytv nov bod tabletu jako linern kombinaci dvou bod
   * @param tp1 TabletPoint
   * @param tp2 TabletPoint
   * @param a double pomr mezi body
   * @param serialNumber int nov seriov slo bodu
   * @return TabletPoint
   */
  private TabletPoint getLinearCombination(TabletPoint tp1, TabletPoint tp2,
                                           double a, int serialNumber) {
    int[] packet = new int[TabletPoint.paket_length];
    packet[TabletPoint.PK_X] = (int) (tp1.getX() + a * (tp2.getX() - tp1.getX()));
    packet[TabletPoint.PK_Y] = (int) (tp1.getY() + a * (tp2.getY() - tp1.getY()));
    packet[TabletPoint.PK_TIME] = (int) (tp1.getTime() +
                                         a * (tp2.getTime() - tp1.getTime()));
    packet[TabletPoint.PK_AZIMUTH] = (int) (tp1.getAzimuth() +
                                            a *
                                            (tp2.getAzimuth() - tp1.getAzimuth()));
    packet[TabletPoint.PK_ALTITUDE] = (int) (tp1.getAltitude() +
                                             a *
                                             (tp2.getAltitude() - tp1.getAltitude()));
    packet[TabletPoint.PK_NORMAL_PRESSURE] = (int) (tp1.getNormalPressure() +
        a * (tp2.getNormalPressure() - tp1.getNormalPressure()));
    packet[TabletPoint.PK_BUTTONS] = tp1.getButtons();
    packet[TabletPoint.PK_SERIAL_NUMBER] = serialNumber;
    return new TabletPoint(packet);
  }

  private double dx[] = null;
  private double dy[] = null;
  private double v[] = null;
  private double vx[] = null;
  private double vy[] = null;
  private Double mv = null;
  private Double mvu = null;

  /**
   * vypot dx, dy, vx, vy, v a prmernou rychlost z dat
   */
  private void computDerivatives() {
    int size = vektor.size();

    double[] nx = SavitzkyGolayFilter.filter(this.getDoubleX(), 3, 5);
    double[] ny = SavitzkyGolayFilter.filter(this.getDoubleY(), 3, 5);

    dx = new double[size];
    dy = new double[size];
    v = new double[size];
    vx = new double[size];
    vy = new double[size];
    mv = new Double(0);

    for (int i = size - 2; i > 0; i--) {
      dx[i] = (double) (nx[i - 1] - nx[i + 1]) / 2;
      dy[i] = (double) (ny[i - 1] - ny[i + 1]) / 2;
      vx[i] = Math.abs(dx[i]) / 5;
      vy[i] = Math.abs(dy[i]) / 5;
      v[i] = Math.sqrt( (dx[i] * dx[i]) + (dy[i] * dy[i])) / 5;
      mv += v[i];
    }

    if(size > 1) {
      v[0] = v[1];
      vx[0] = vx[1];
      vy[0] = vy[1];
      v[size - 1] = v[size - 2];
      vx[size - 1] = vx[size - 2];
      vy[size - 1] = vy[size - 2];
    }
    mvu = (mv + 10000) / (size + 1);
    mv /= size;
  }

  /**
   * Vypot vektor hl, kter jednotliv body svraj s osou x
   * @return double[]
   */
  public double[] getAngle() {
    if ( (dx == null) || (dy == null)) {
      this.computDerivatives();
    }
    return MathUtil.getFi(dx, dy, true);
  }

  /**
   * Vrac prmrnou rychlost kreslen obrazce
   * @return double
   */
  public double getMeanVelocityUpdated() {
    if (mvu == null) {
      this.computDerivatives();
    }
    return mvu;
  }

  /**
   * vrac vektor okamitch rychlost
   * @return double[]
   */
  public double[] getVelocity() {
    if (v == null) {
      this.computDerivatives();
    }
    return v;
  }

  /**
   * vrac vektor okamitch rychlost ve smru osy x
   * @return double[]
   */
  public double[] getVelocityX() {
    if (vx == null) {
      this.computDerivatives();
    }
    return vx;
  }

  /**
   * vrac vektor okamitch rychlost ve smru osy y
   * @return double[]
   */
  public double[] getVelocityY() {
    if (vy == null) {
      this.computDerivatives();
    }
    return vy;
  }
}
