/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import spirals.motol.loaders.XML.XMLDatabaseLoader;
import spirals.motol.database.*;
import java.io.*;
import java.util.zip.*;
import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * <p>Main Frame of aplication</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class MainFrame
    extends JFrame {
  /**
   * engine
   */
  private Engine engine;
  /**
   * minimal width of frame
   */
  private final int minWidth = 750;
  /**
   * minimal height of frame
   */
  private final int minHeight = 600;
  /**
   * small font for components
   */
  private Font smallFont;
  /**
   * default (medium) font for components
   */
  private Font defaultFont;
  /**
   * default (medium) bold font for components
   */
  private Font defaultBoldFont;
  /**
   * big font for components
   */
  private Font bigFont;
  /**
   * tree with pacients and visits
   */
  private PacientTree tree;

  /**
   * contains content of frame
   */
  private JPanel contentPane;
  /**
   * container for changable panels
   */
  private JPanel rightPanel = new JPanel(new BorderLayout());
  /**
   * status bar
   */
  private JLabel statusBar = new JLabel(" ");

  /**
   * exit button
   */
  private MainFrame_Exit_Action exitAction = new MainFrame_Exit_Action(this);
  /**
   * about button
   */
  private MainFrame_Help_Action aboutHelpAction = new MainFrame_Help_Action(this);
  /**
   * add pacient button
   */
  private MainFrame_AddPacient_Action addPacientAction = new
      MainFrame_AddPacient_Action(this);
  /**
   * export button
   */
  private MainFrame_Export_Action exportAction = new MainFrame_Export_Action(this);
  private MainFrame_Import_Action importAction = new MainFrame_Import_Action(this);
  /**
   * configure button
   */
  private MainFrame_Configure_Action configureAction = new
      MainFrame_Configure_Action(this);
  /**
   * backup button
   */
  private MainFrame_Backup_Action backupAction = new MainFrame_Backup_Action(this);

  /**
   * create new frame with Configuration
   */
  public MainFrame(Configuration conf) {
    this.engine = Engine.getInstance(this, conf);
    this.createFonts();
    try {
      setDefaultCloseOperation(DISPOSE_ON_CLOSE);
      jbInit();
    }
    catch (Exception exception) {
      exception.printStackTrace();
    }
  }

  /**
   * initializes fonts for components
   */
  private void createFonts() {
    smallFont = new Font("Tahoma", Font.PLAIN, 12);
    defaultFont = new Font("Tahoma", Font.PLAIN, 13);
    defaultBoldFont = new Font("Tahoma", Font.BOLD, 13);
    bigFont = new Font("Tahoma", Font.PLAIN, 14);
  }

  /**
   * Component initialization.
   * @throws java.lang.Exception
   */
  private void jbInit() throws Exception {
    this.setSize(new Dimension(minWidth, minHeight));
    this.setTitle("Parkinson Disease Database");
    this.setResizable(false);
    this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    this.addWindowListener(exitAction);
    this.setFont(this.getDefaultFont());
    this.setIconImage( (new ImageIcon(MainFrame.class.getResource(
        "/res/icon.gif"))).getImage());
    contentPane = (JPanel) getContentPane();

    createMenu();
    createToolbar();
    createContent();
    createStatusBar();
    this.setRightPanel(new DatabasePanel(engine));
  }

  /**
   * lay out menu bar
   */
  private void createMenu() {
    JMenuBar menuBar = new JMenuBar();
    JMenu jMenuFile = new JMenu("Soubor");
    jMenuFile.setMnemonic(KeyEvent.VK_S);
    JMenuItem jMenuFileExit = new JMenuItem(exitAction);
    jMenuFileExit.setIcon(null);
    JMenu jMenuHelp = new JMenu("Npovda");
    jMenuHelp.setMnemonic(KeyEvent.VK_V);
    JMenuItem jMenuHelpAbout = new JMenuItem(aboutHelpAction);
    jMenuHelpAbout.setIcon(null);

    menuBar.add(jMenuFile);
    jMenuFile.add(jMenuFileExit);
    menuBar.add(jMenuHelp);
    jMenuHelp.add(jMenuHelpAbout);
    setJMenuBar(menuBar);
  }

  /**
   * lay out top tool bar
   */
  private void createToolbar() {
    JToolBar toolBar = new JToolBar();
    toolBar.setFloatable(false);
    JToolBar jToolBar1 = new JToolBar();

    createButton(jToolBar1, addPacientAction);
    jToolBar1.addSeparator();
    createButton(jToolBar1, exportAction);
    createButton(jToolBar1, importAction);
    createButton(jToolBar1, backupAction);
    createButton(jToolBar1, configureAction);
    jToolBar1.addSeparator();
    createButton(jToolBar1, aboutHelpAction);
    jToolBar1.addSeparator();
    createButton(jToolBar1, exitAction);

    toolBar.add(jToolBar1);
    contentPane.add(toolBar, BorderLayout.NORTH);
  }

  /**
   * create button from Action for ToolBar
   */
  private void createButton(JToolBar toolBar, Action action) {
    JButton res = new JButton(action);
    res.setVerticalTextPosition(SwingConstants.BOTTOM);
    res.setHorizontalTextPosition(SwingConstants.CENTER);
    res.setFont(this.getSmallFont());
    toolBar.add(res);
  }

  /**
   * create main (center) content
   */
  private void createContent() {
    tree = new PacientTree(this);
    JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tree,
                                      rightPanel);
    tree.setBorder(BorderFactory.createEtchedBorder());
    rightPanel.setBorder(BorderFactory.createEtchedBorder());
    split.setDividerSize(0);
    split.setDividerLocation(200);
    contentPane.add(split, java.awt.BorderLayout.CENTER);
  }

  /**
   * create status bar
   */
  private void createStatusBar() {
    contentPane.add(statusBar, BorderLayout.SOUTH);
    statusBar.setFont(this.getBigFont());
  }

  /**
   * return small font
   */
  public Font getSmallFont() {
    return smallFont;
  }

  /**
   * return default (medium) font
   */
  public Font getDefaultFont() {
    return defaultFont;
  }

  /**
   * return default (medium) bold font
   */
  public Font getDefaultBoldFont() {
    return defaultBoldFont;
  }

  /**
   * return big font
   */
  public Font getBigFont() {
    return bigFont;
  }

  /**
   * return engine
   */
  public Engine getEngine() {
    return engine;
  }

  /**
   * set status bar text
   */
  public void setStatus(String text) {
    statusBar.setText(text);
  }

  /**
   * change right panel
   */
  public boolean setRightPanel(RemovablePanel panel) {
    Component[] list = rightPanel.getComponents();
    if (list.length > 1) {
      System.err.println("Moc komponent");
    }
    else {
      try {
        if (list.length != 0) {
          if (! ( (RemovablePanel) list[0]).notifyRemoved()) { // let old panel known about removing
            return false;
          }
        }
        rightPanel.removeAll();
        if (panel != null) {
          rightPanel.add(panel, java.awt.BorderLayout.CENTER);
          rightPanel.validate();
//          panel.revalidate();
        }
      }
      catch (ClassCastException e) {
        System.err.println("Podivna komponenta");
        e.printStackTrace();
      }
    }
    return true;
  }

  /**
   * return tree vith pacients and visits
   */
  public PacientTree getTree() {
    return tree;
  }
}

/**
 * <p>Action performed when about button is clicked.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MainFrame_Help_Action
    extends AbstractAction {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_Help_Action(MainFrame adaptee) {
    super("O aplikaci",
          new ImageIcon(MainFrame.class.getResource("/res/about.gif")));
    putValue(SHORT_DESCRIPTION, "O aplikaci");
    putValue(MNEMONIC_KEY, KeyEvent.VK_A);
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent actionEvent) {
    MainFrame_AboutBox dlg = new MainFrame_AboutBox(adaptee);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = adaptee.getSize();
    Point loc = adaptee.getLocation();
    dlg.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                    (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.setVisible(true);
  }
}

/**
 * <p>Action performed when exit button is clicked or window is closing.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MainFrame_Exit_Action
    extends AbstractAction implements WindowListener {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_Exit_Action(MainFrame adaptee) {
    super("Ukoni", new ImageIcon(MainFrame.class.getResource("/res/exit.gif")));
    putValue(SHORT_DESCRIPTION, "Ukoni aplikaci");
    putValue(MNEMONIC_KEY, KeyEvent.VK_U);
    this.adaptee = adaptee;
  }

  /**
   * for the sake of Java
   */
  public void windowActivated(WindowEvent e) {}

  /**
   * for the sake of Java
   */
  public void windowClosed(WindowEvent e) {}

  /**
   * for the sake of Java
   */
  public void windowDeactivated(WindowEvent e) {}

  /**
   * for the sake of Java
   */
  public void windowDeiconified(WindowEvent e) {}

  /**
   * for the sake of Java
   */
  public void windowIconified(WindowEvent e) {}

  /**
   * for the sake of Java
   */
  public void windowOpened(WindowEvent e) {}

  /**
   * called when window is closing
   */
  public void windowClosing(WindowEvent e) {
    this.exit();
  }

  /**
   * called when exit button is clicked
   */
  public void actionPerformed(ActionEvent e) {
    this.exit();
  }

  /**
   * perform clean exit
   */
  private void exit() {
    if (adaptee.setRightPanel(null)) {
      adaptee.getEngine().getTablet().close();
      System.exit(0);
    }
  }
};

/**
 * <p>Action performed when add pacient button is clicked.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MainFrame_AddPacient_Action
    extends AbstractAction {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_AddPacient_Action(MainFrame adaptee) {
    super("Nov pacient",
          new ImageIcon(MainFrame.class.getResource("/res/add.gif")));
    putValue(SHORT_DESCRIPTION, "Zalo novho pacienta");
    putValue(MNEMONIC_KEY, KeyEvent.VK_N);
    this.adaptee = adaptee;
  }

  /**
   * called when add pacient button is clicked
   */
  public void actionPerformed(ActionEvent actionEvent) {
    adaptee.getTree().clearSelection();
    adaptee.setRightPanel(new PacientPanel(adaptee,
                                           new Pacient(adaptee.getEngine().
        getDatabase()), true));
  }
}

/**
 * <p>Action performed when config button is clicked.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MainFrame_Configure_Action
    extends AbstractAction {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_Configure_Action(MainFrame adaptee) {
    super("Konfigurace",
          new ImageIcon(MainFrame.class.getResource("/res/configure.gif")));
    putValue(SHORT_DESCRIPTION, "Konfigurace");
    putValue(MNEMONIC_KEY, KeyEvent.VK_C);
    this.adaptee = adaptee;
  }

  /**
   * called when config button is clicked
   */
  public void actionPerformed(ActionEvent actionEvent) {
    ConfigFrame dlg = new ConfigFrame(adaptee.getEngine());
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = adaptee.getSize();
    Point loc = adaptee.getLocation();
    dlg.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                    (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.pack();
    dlg.setVisible(true);
  }
}

/**
 * <p>Action performed when export button is clicked.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MainFrame_Export_Action
    extends AbstractAction {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_Export_Action(MainFrame adaptee) {
    super("Export", new ImageIcon(MainFrame.class.getResource("/res/export.gif")));
    putValue(SHORT_DESCRIPTION, "Exportuj data");
    putValue(MNEMONIC_KEY, KeyEvent.VK_E);
    this.adaptee = adaptee;
  }

  /**
   * called when export button is clicked
   */
  public void actionPerformed(ActionEvent actionEvent) {
    adaptee.getEngine().getDatabase().export(adaptee.getEngine().getConfiguration().getDatabaseLoader());
  }
}

class MainFrame_Import_Action
    extends AbstractAction {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_Import_Action(MainFrame adaptee) {
    super("Import", new ImageIcon(MainFrame.class.getResource("/res/import.gif")));
    putValue(SHORT_DESCRIPTION, "Importuj data");
    putValue(MNEMONIC_KEY, KeyEvent.VK_I);
    this.adaptee = adaptee;
  }

  /**
   * called when export button is clicked
   */
  public void actionPerformed(ActionEvent actionEvent) {
    adaptee.getEngine().getDatabase().importData(new XMLDatabaseLoader(new File(adaptee.getEngine().getConfiguration().getDatabaseRoot(), "../data.temp")));
  }
}

/**
 * <p>Action performed when backup button is clicked.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
class MainFrame_Backup_Action
    extends AbstractAction {
  /**
   * parent MainFrame
   */
  MainFrame adaptee;

  /**
   * create action for MainFrame
   */
  MainFrame_Backup_Action(MainFrame adaptee) {
    super("Zloha", new ImageIcon(MainFrame.class.getResource("/res/backup.gif")));
    putValue(SHORT_DESCRIPTION, "Zlohuj databzi");
    putValue(MNEMONIC_KEY, KeyEvent.VK_B);
    this.adaptee = adaptee;
  }

  /**
   * called when backup button is clicked
   */
  public void actionPerformed(ActionEvent actionEvent) {
    String[] options = {
        "zip souboru", "adrese"};
    String option = (String) JOptionPane.showInputDialog(adaptee, "Uloit do",
        "Zloha databze", JOptionPane.INFORMATION_MESSAGE, null, options,
        options[0]);
    if (option != null) {
      boolean adr = option.equals(options[0]);
      if (adr && !backupZip()) {
        return;
      }
      else if (!adr && !backupFolder()) {
        return;
      }
      System.err.println("Databaze zalohovana");
      JOptionPane.showMessageDialog(adaptee,
                                    "Zloha databze probhla",
                                    "Zloha databze",
                                    JOptionPane.INFORMATION_MESSAGE);
      adaptee.getEngine().getDatabase().backuped();
    }
  }

  /**
   * backup database as directory
   * @return success
   */
  private boolean backupFolder() {
    JFileChooser fc = new JFileChooser();
    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    if (fc.showSaveDialog(adaptee) == JFileChooser.APPROVE_OPTION) {
      try {
        XMLDatabaseLoader.copyDir( ( (XMLDatabaseLoader) adaptee.getEngine().
                                    getDatabase().getLoader()).getRootFile(),
                                  new File(fc.getSelectedFile(), "database"));
        return true;
      }
      catch (IOException e) {
        System.err.println("Nelze zlohovat databazi");
      }
    }
    return false;
  }

  /**
   * backup database as archive
   * @return success
   */
  private boolean backupZip() {
    JFileChooser fc = new JFileChooser();
    Date now = new Date();
    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    fc.setSelectedFile(new File("database-" + df.format(now) + ".zip"));
    if (fc.showSaveDialog(adaptee) == JFileChooser.APPROVE_OPTION) {
      File outFile = fc.getSelectedFile();
      ZipOutputStream out = null;
      try {
        out = new ZipOutputStream(new FileOutputStream(outFile));
        out.setMethod(ZipOutputStream.DEFLATED);
        out.setLevel(6);
        XMLDatabaseLoader.zipDir( ( (XMLDatabaseLoader) adaptee.getEngine().
                                   getDatabase().getLoader()).getRootFile(),
                                 out, new File(""));
        return true;
      }
      catch (IOException e) {
        System.err.println("Nelze zlohovat databazi");
      }
      finally {
        if (out != null) {
          try {
            out.close();
          }
          catch (IOException e) {}
        }
      }
    }
    return false;
  }
}
