/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.loaders.XML;

import spirals.motol.loaders.*;
import spirals.motol.database.*;
import spirals.data.TabletData;
import java.text.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import org.jdom.*;
import tablet.*;

/**
 * <p>Tda, kter se star o nahrn dat z XML loit.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class XMLDataLoader
    extends DataLoader {
  public static final String dataTag = "data";
  public static final String descAttr = "desc";
  public static final String tabletTag = "tablet";
  public static final String nameAttr = "name";
  public static final String minxAttr = "minx";
  public static final String maxxAttr = "maxx";
  public static final String minyAttr = "miny";
  public static final String maxyAttr = "maxy";
  public static final String startAttr = "start";
  public static final String endAttr = "end";
  public static final String fileAttr = "file";
  public static final String memoTag = "memo";
  public static final String dateFormat = "yyyy_MM_dd_HH_mm_ss";

  protected XMLDataLoader() {}

  public static DataLoader getInstance() {
    if (instance == null) {
      instance = new XMLDataLoader();
    }
    return instance;
  }

  public void load(Data data, Visit visit, Date date) {
    try {
      DateFormat df = new SimpleDateFormat(dateFormat);
      String dataFile = df.format(date) + ".zip";

      Document doc = XMLVisitLoader.builder.build(XMLVisitLoader.getFile(visit, true));
      Element root = doc.getRootElement();
      Iterator i = root.getDescendants();

      while (i.hasNext()) {
        Object o = i.next();
        if (o instanceof Element) {
          Element e = (Element) o;
          String eFile = e.getAttributeValue(fileAttr);
          if ( (eFile != null) && eFile.equals(dataFile)) {
            this.setData(data, e);
            break;
          }
        }
      }
    }
    catch (JDOMException e) {
      System.err.println("Chyba parsovani souboru databaze.");
      System.err.println(e.getMessage());
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * Pro danou nvtvu nahraje popis dat z XML elementu
   * @param visit Visit
   * @param e Element
   * @return Data
   */
  public Data load(Visit visit, Element e) {
    Data d = new Data(visit);
    this.setData(d, e);
    return d;
  }

  /**
   * Zape obsah XML elementu do dat
   * @param d Data
   * @param e Element
   */
  private void setData(Data d, Element e) {
    d.setTabletInfo(getTabletInfoFromTag(e.getChild(tabletTag)));
    d.setDescription(e.getAttributeValue(descAttr));
    d.setMemo(e.getChildTextTrim(memoTag));
    d.setStart(getInteger(e.getAttributeValue(startAttr)));
    d.setEnd(getInteger(e.getAttributeValue(endAttr)));

    DateFormat df = new SimpleDateFormat(XMLDataLoader.dateFormat + "'.zip'");
    try {
      d.setDate(df.parse(e.getAttributeValue(XMLDataLoader.fileAttr)));
    }
    catch (ParseException ex) {
      System.err.println("Chyba parsovani dat");
    }
  }

  /**
   * Vytvo z obsahu XML elementu tdu TabletInfo
   * @param tabEl Element
   * @return TabletInfo
   */
  private static final TabletInfo getTabletInfoFromTag(Element tabEl) {
    int minx = Integer.parseInt(tabEl.getAttributeValue(minxAttr));
    int maxx = Integer.parseInt(tabEl.getAttributeValue(maxxAttr));
    int miny = Integer.parseInt(tabEl.getAttributeValue(minyAttr));
    int maxy = Integer.parseInt(tabEl.getAttributeValue(maxyAttr));
    return new TabletInfo(tabEl.getAttributeValue(nameAttr), minx, maxx, miny,
                          maxy);
  }

  /**
   * Pokus se pevst etzec na integer
   * @param str String
   * @return int
   */
  private static int getInteger(String str) {
    int res = 0;
    try {
      if (str != null) {
        return Integer.parseInt(str);
      }
    }
    catch (NumberFormatException e) {
      System.err.println("Chyba parsovani rozsahu dat");
    }
    return res;
  }

  public void save(Data data, boolean saveData) {
    XMLVisitLoader.getInstance().save(data.getVisit());
    if (saveData) {
      this.saveData(data);
    }
  }

  /**
   * Pevede popis dat na XML element
   * @param d Data
   * @return Element
   */
  public static Element getElement(Data d) {
    Element dataEl = new Element(dataTag);
    dataEl.setAttribute(descAttr, d.getDescription());
    dataEl.setAttribute(fileAttr, getFileName(d));
    dataEl.setAttribute(startAttr, "" + d.getStart());
    dataEl.setAttribute(endAttr, "" + d.getEnd());
    Element tabEl = new Element(tabletTag);
    TabletInfo ti = d.getTabletInfo();
    tabEl.setAttribute(nameAttr, ti.getTabletName());
    tabEl.setAttribute(minxAttr, "" + ti.getMinX());
    tabEl.setAttribute(maxxAttr, "" + ti.getMaxX());
    tabEl.setAttribute(minyAttr, "" + ti.getMinY());
    tabEl.setAttribute(maxyAttr, "" + ti.getMaxY());
    Element memoEl = new Element(memoTag);
    memoEl.setText(d.getMemo());
    dataEl.addContent(memoEl);
    dataEl.addContent(tabEl);
    return dataEl;
  }

  /**
   * nahraje nasnman data ze souboru
   * @param data Data
   * @return TabletData
   */
  public TabletData getData(Data data) {
    TabletData td = new TabletData(data);
    ZipInputStream in = null;
    try {
      in = new ZipInputStream(new FileInputStream(this.getFile(data, true)));
      in.getNextEntry(); // first and only entry in zip file
      byte buff[] = new byte[20];
      int len = 0;
      while ( (len = in.read(buff)) > -1) {
        while (len != 20) { // read until all twenty bytes of one point are collected
          byte array[] = new byte[20 - len];
          len += in.read(array);
          System.arraycopy(array, 0, buff, 20 - array.length, array.length);
        }
        td.add(new TabletPoint(buff));
      }
      in.closeEntry();
    }
    catch (FileNotFoundException e) {
      // vytvarime novy soubor
    }
    catch (IOException e) {
      System.err.println("Chyba nacitani dat");
      e.printStackTrace();
    }
    finally {
      if (in != null) {
        try {
          in.close();
        }
        catch (IOException e) {}
      }
    }
    return td;
  }

  /**
   * ulo nasnman data do souboru
   * @param data Data
   */
  private void saveData(Data data) {
    ZipOutputStream out = null;
    try {
      File file = this.getFile(data, false);
      out = new ZipOutputStream(new FileOutputStream(file));
      out.setMethod(ZipOutputStream.DEFLATED);
      out.setLevel(6);
      String name = file.getName().substring(0, file.getName().indexOf(".zip")) +
          ".dat";
      ZipEntry anEntry = new ZipEntry(name);
      out.putNextEntry(anEntry);
      for (TabletPoint tp : data.getAllData().getData()) {
        tp.print(out);
      }
    }
    catch (IOException e) {
      System.err.println("Nepodarilo se zapsat data");
      e.printStackTrace();
    }
    finally {
      if (out != null) {
        try {
          out.close();
        }
        catch (IOException e) {}
      }
    }
  }

  /**
   * Vytvo cestu pro data a pacienta
   * @param data Data
   * @param ID String ID pacienta, ke ktermu data pat / maj patit
   * @param load boolean true, pokud nahrvm data z temp databze pi sluovn
   * @return File cesta k souboru s daty
   */
  public static File getFile(Data data, String ID, boolean load) {
    return new File(new File(getRootFile(data, load), ID), getFileName(data));
  }

  /**
   * Vytvo cestu pro data
   * @param data Data
   * @param load boolean true, pokud nahrvm data z temp databze pi sluovn
   * @return File cesta k souboru s daty
   */
  public static File getFile(Data data, boolean load) {
    return getFile(data, data.getVisit().getPacient().getID(), load);
  }

  /**
   * Vrt jmno souboru s daty
   * @param data Data
   * @return String
   */
  public static String getFileName(Data data) {
    SimpleDateFormat df = new SimpleDateFormat(dateFormat);
    return df.format(data.getDate()) + ".zip";
  }

  /**
   * zjisti adres ve kterm je soubor s daty uloen
   * @param data Data
   * @param load boolean true, pokud nahrvm data z temp databze pi sluovn
   * @return File
   */
  public static File getRootFile(Data data, boolean load) {
    return XMLVisitLoader.getRootFile(data.getVisit(), load);
  }
}
